@@ -54,7 +54,7 @@ def get_mat(rotation=180.0, shear=2.0, height_zoom=8.0, width_zoom=8.0, height_s
                  K.dot(zoom_matrix, shift_matrix))
 
 
-def transform(image, dimension, rotate=180.0, shear=2.0, hzoom=8.0, vzoom=8.0, hshift=8.0, wshift=8.0):
+def transform(image, dimension, rotate=180.0, shear=2.0, hzoom=8.0, vzoom=8.0, hshift=8.0, wshift=8.0, prob=0.5):
     """ transforms an image by rotating, zooming a shearing
 
     Input:
@@ -66,11 +66,16 @@ def transform(image, dimension, rotate=180.0, shear=2.0, hzoom=8.0, vzoom=8.0, h
         width_zoom: width zoom ratio
         height_shift: height shift ratio
         width_shift: width shift ratio
+        prob: probability to perform transformation
 
     Output:
         image randomly rotated, sheared, zoomed, and shifted
     """
 
+    P = tf.cast(tf.random.uniform([], 0, 1) < prob, tf.int32)
+    if P == 0:
+        return image  # no action
+
     DIM = dimension
     XDIM = DIM % 2  # fix for size 331
 
@@ -145,6 +150,98 @@ def dropout(image, height=256, width=256, prob=0.75, ct=8, sz=0.2):
     image = tf.reshape(image, [height, width, 3])
     return image
 
+def mixup_labels(label1, label2, n_classes, a):
+    if len(label1.shape) == 1:
+        lab1 = tf.one_hot(label1, n_classes)
+        lab2 = tf.one_hot(label2, n_classes)
+    else:
+        lab1 = label1
+        lab2 = label2
+    return (1 - a) * lab1 + a * lab2
+
+def cutmix(image, label, batch_size=16, dimension=256, n_classes = 1, prob=1.0):
+    # input image - is a batch of images of size [n,dim,dim,3] not a single image of [dim,dim,3]
+    # output - a batch of images with cutmix applied
+    P = tf.cast(tf.random.uniform([], 0, 1) < prob, tf.int32)
+    if (P == 0):
+        return image  # no action
+
+    DIM = dimension
+
+    imgs = [];
+    labs = []
+    for j in range(batch_size):
+        # DO CUTMIX WITH PROBABILITY DEFINED ABOVE
+        P = tf.cast(tf.random.uniform([], 0, 1) <= prob, tf.int32)
+        # CHOOSE RANDOM IMAGE TO CUTMIX WITH
+        k = tf.cast(tf.random.uniform([], 0, batch_size), tf.int32)
+        # CHOOSE RANDOM LOCATION
+        x = tf.cast(tf.random.uniform([], 0, DIM), tf.int32)
+        y = tf.cast(tf.random.uniform([], 0, DIM), tf.int32)
+        b = tf.random.uniform([], 0, 1)  # this is beta dist with alpha=1.0
+        WIDTH = tf.cast(DIM * tf.math.sqrt(1 - b), tf.int32) * P
+        ya = tf.math.maximum(0, y - WIDTH // 2)
+        yb = tf.math.minimum(DIM, y + WIDTH // 2)
+        xa = tf.math.maximum(0, x - WIDTH // 2)
+        xb = tf.math.minimum(DIM, x + WIDTH // 2)
+        # MAKE CUTMIX IMAGE
+        one = image[j, ya:yb, 0:xa, :]
+        two = image[k, ya:yb, xa:xb, :]
+        three = image[j, ya:yb, xb:DIM, :]
+        middle = tf.concat([one, two, three], axis=1)
+        img = tf.concat([image[j, 0:ya, :, :], middle, image[j, yb:DIM, :, :]], axis=0)
+        imgs.append(img)
+        # MAKE CUTMIX LABEL
+        a = tf.cast(WIDTH * WIDTH / DIM / DIM, tf.float32)
+        if len(label.shape) == 1:
+            lab1 = tf.one_hot(label[j], n_classes)
+            lab2 = tf.one_hot(label[k], n_classes)
+        else:
+            lab1 = label[j,]
+            lab2 = label[k,]
+        labs.append((1 - a) * lab1 + a * lab2)
+
+    # RESHAPE HACK SO TPU COMPILER KNOWS SHAPE OF OUTPUT TENSOR (maybe use Python typing instead?)
+    image2 = tf.reshape(tf.stack(imgs),(batch_size, DIM, DIM, 3))
+    label2 = tf.reshape(tf.stack(labs),(batch_size, n_classes))
+    return image2,label2
+
+def mixup(image, label, batch_size=16, dimension=256, n_classes=1, prob=1.0):
+    # input image - is a batch of images of size [n,dim,dim,3] not a single image of [dim,dim,3]
+    # output - a batch of images with mixup applied
+    P = tf.cast(tf.random.uniform([], 0, 1) < prob, tf.int32)
+    if (P == 0):
+        return image  # no action
+
+    DIM = dimension
+    CLASSES = n_classes
+
+    imgs = []
+    labs = []
+    for j in range(batch_size):
+        # DO MIXUP WITH PROBABILITY DEFINED ABOVE
+        P = tf.cast(tf.random.uniform([], 0, 1) <= prob, tf.float32)
+        # CHOOSE RANDOM
+        k = tf.cast(tf.random.uniform([], 0, batch_size), tf.int32)
+        a = tf.random.uniform([], 0, 1) * P  # this is beta dist with alpha=1.0
+        # MAKE MIXUP IMAGE
+        img1 = image[j,]
+        img2 = image[k,]
+        imgs.append((1 - a) * img1 + a * img2)
+        # MAKE CUTMIX LABEL
+        if len(label.shape) == 1:
+            lab1 = tf.one_hot(label[j], CLASSES)
+            lab2 = tf.one_hot(label[k], CLASSES)
+        else:
+            lab1 = label[j,]
+            lab2 = label[k,]
+        labs.append((1 - a) * lab1 + a * lab2)
+
+    # RESHAPE HACK SO TPU COMPILER KNOWS SHAPE OF OUTPUT TENSOR (maybe use Python typing instead?)
+    image2 = tf.reshape(tf.stack(imgs), (batch_size, DIM, DIM, 3))
+    label2 = tf.reshape(tf.stack(labs), (batch_size, CLASSES))
+    return image2, label2
+
 
 def spec_augmentation(image, prob=0.66, time_drop_width=0.0625, time_stripes_num=2, freq_drop_width=0.125,
                       freq_stripes_num=2, height=64, width=501):
